@@ -1,4 +1,4 @@
-/* Copyright (C) 2008-2021 Free Software Foundation, Inc.
+/* Copyright (C) 2008-2022 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -436,7 +436,7 @@ static const struct lval_funcs tlb_value_funcs =
 static struct value *
 tlb_make_value (struct gdbarch *gdbarch, struct internalvar *var, void *ignore)
 {
-  if (target_has_stack && inferior_ptid != null_ptid)
+  if (target_has_stack () && inferior_ptid != null_ptid)
     {
       struct type *type = windows_get_tlb_type (gdbarch);
       return allocate_computed_value (type, &tlb_value_funcs, NULL);
@@ -488,7 +488,7 @@ display_one_tib (ptid_t ptid)
       return -1;
     }
 
-  if (target_read (current_top_target (), TARGET_OBJECT_MEMORY,
+  if (target_read (current_inferior ()->top_target (), TARGET_OBJECT_MEMORY,
 		   NULL, tib, thread_local_base, tib_size) != tib_size)
     {
       printf_filtered (_("Unable to read thread information "
@@ -617,7 +617,7 @@ init_w32_command_list (void)
       add_basic_prefix_cmd
 	("w32", class_info,
 	 _("Print information specific to Win32 debugging."),
-	 &info_w32_cmdlist, "info w32 ", 0, &infolist);
+	 &info_w32_cmdlist, 0, &infolist);
       w32_prefix_command_valid = 1;
     }
 }
@@ -759,13 +759,13 @@ create_enum (struct gdbarch *gdbarch, int bit, const char *name,
   type->set_num_fields (count);
   type->set_fields
     ((struct field *) TYPE_ZALLOC (type, sizeof (struct field) * count));
-  TYPE_UNSIGNED (type) = 1;
+  type->set_is_unsigned (true);
 
   for (i = 0; i < count; i++)
-  {
-    TYPE_FIELD_NAME (type, i) = values[i].name;
-    SET_FIELD_ENUMVAL (type->field (i), values[i].value);
-  }
+    {
+      TYPE_FIELD_NAME (type, i) = values[i].name;
+      SET_FIELD_ENUMVAL (type->field (i), values[i].value);
+    }
 
   return type;
 }
@@ -903,7 +903,7 @@ windows_solib_create_inferior_hook (int from_tty)
     }
   CORE_ADDR tlb;
   gdb_byte buf[8];
-  if (target_has_execution
+  if (target_has_execution ()
       && target_get_tib_address (inferior_ptid, &tlb)
       && !target_read_memory (tlb + peb_offset, buf, ptr_bytes))
     {
@@ -913,11 +913,13 @@ windows_solib_create_inferior_hook (int from_tty)
     }
 
   /* Rebase executable if the base address changed because of ASLR.  */
-  if (symfile_objfile != nullptr && exec_base != 0)
+  if (current_program_space->symfile_object_file != nullptr && exec_base != 0)
     {
-      CORE_ADDR vmaddr = pe_data (exec_bfd)->pe_opthdr.ImageBase;
+      CORE_ADDR vmaddr
+	= pe_data (current_program_space->exec_bfd ())->pe_opthdr.ImageBase;
       if (vmaddr != exec_base)
-	objfile_rebase (symfile_objfile, exec_base - vmaddr);
+	objfile_rebase (current_program_space->symfile_object_file,
+			exec_base - vmaddr);
     }
 }
 
@@ -1065,7 +1067,7 @@ section's range [0x%" BFD_VMA_FMT "x, 0x%" BFD_VMA_FMT "x[."),
       bfd_vma name_va = dir_entry->name_rva;
 
       /* If the name's virtual address is smaller than the section's virtual
-         address, there's a problem.  */
+	 address, there's a problem.  */
       if (name_va < idata_section_va || name_va >= idata_section_end_va)
 	{
 	  warning (_("\
@@ -1211,10 +1213,11 @@ _initialize_windows_tdep ()
     = gdbarch_data_register_post_init (init_windows_gdbarch_data);
 
   init_w32_command_list ();
-  add_cmd ("thread-information-block", class_info, display_tib,
-	   _("Display thread information block."),
-	   &info_w32_cmdlist);
-  add_alias_cmd ("tib", "thread-information-block", class_info, 1,
+  cmd_list_element *info_w32_thread_information_block_cmd
+    = add_cmd ("thread-information-block", class_info, display_tib,
+	       _("Display thread information block."),
+	       &info_w32_cmdlist);
+  add_alias_cmd ("tib", info_w32_thread_information_block_cmd, class_info, 1,
 		 &info_w32_cmdlist);
 
   add_setshow_boolean_cmd ("show-all-tib", class_maintenance,
@@ -1463,6 +1466,7 @@ cygwin_sigwrapper_frame_sniffer (const struct frame_unwind *self,
 
 const struct frame_unwind cygwin_sigwrapper_frame_unwind =
 {
+  "cygwin sigwrapper",
   NORMAL_FRAME,
   default_frame_unwind_stop_reason,
   &cygwin_sigwrapper_frame_this_id,
